HEAP_SIZE = 0x1000;
STACK_SIZE = 0x1000;


ENTRY(reset)

MEMORY
{
	FLASH : ORIGIN = 0x0000100, LENGTH = 0x00100000
	RAM   : ORIGIN = 0x00200000, LENGTH = 0x00100000
}

SECTIONS
{
	.vectors 0x0 : {
		*(.vectors)
	}
	
	.text : {
		__text_start__ = .;
		*(.text)
		*(.text.*)
		*(.gnu.linkonce.t.*)
		*(.glue_7)
		*(.glue_7t)
		__text_end__ = .;
	} > FLASH
	
	.rodata : {
		__rodata_start__ = .;
		*(.rodata)
		*(.rodata.*)
		*(.gnu.linkonce.r.*)
		__rodata_end__ = .;
	} > FLASH
	
	.ctors : {
		__ctors_start__ = .;
		KEEP (*(SORT(.ctors.*)))
		KEEP (*(.ctors))
		__ctors_end__ = .;
	} > FLASH

	.dtors : {
		__dtors_start__ = .;
		KEEP (*(SORT(.dtors.*)))
		KEEP (*(.dtors))
		__dtors_end__ = .;
	} > FLASH

	.jcr : {
		*(.jcr)
	} > FLASH
	
	.init : {
		*(.init)
	} > FLASH
	
	.fini : {
		*(.fini)
	} > FLASH
	
	.eh_frame : {
		*(.eh_frame)
	} > FLASH
	
	.data : {
		. = ALIGN(4);
		__data_start__ = .;
		*(.data)
		*(.gnu.linkonce.d.*)
		. = ALIGN(4);
		__data_end__ = .;
	} > RAM AT > FLASH
	
	__data_lma__ = LOADADDR(.data);
	
	.bss : {
		. = ALIGN(4);
		__bss_start__ = .;
		*(.bss)
		*(.gnu.linkonce.b.*)
		. = ALIGN(4);
		__bss_end__ = .;
	} > RAM
	
	
	
	.heap :	{
		. = ALIGN(4);
		PROVIDE ( end = . );
		__heap_start__ = .;
		. += HEAP_SIZE;
		. = ALIGN(4);
		__heap_end__ = .;
	} > RAM
	
	.stack : {
		. = ALIGN(4);
		__stack_end__ = .;
		. += STACK_SIZE;
		. = ALIGN(4);
		__stack_start__ = .;		/* NOTE: stack increases to lower adresses */
	} > RAM
}
