.global reset
.global exit

.section .vectors
.arm											/* use ARM instructions (not THUMB) here */

reset:			ldr		pc, .Lvector_reset		/* RESET */
				b exit							/* UNDEFINED INSTRUCTION */
				ldr		pc, .Lvector_swi		/* SWI */
				b exit 							/* PREFETCH ABORT */
				b exit							/* DATA ABORT */
				nop								/* RESERVED */
				b exit
				b exit 							/* FIQ */

.Lvector_reset:		.word	.Lstartup		/* RESET: executed after processor reset or power-up */
.Lvector_undef:		.word	0				/* UNDEFINED INSTRUCTION */
.Lvector_swi:		.word	.Lswi_isr		/* SWI */
.Lvector_pref_abt:	.word	0				/* PREFETCH ABORT */
.Lvector_data_abt:	.word	0				/* DATA ABORT */
					.word	0
.Lvector_fiq:		.word	0				/* FIQ */
                	.word	0				/* round the vectors and ISR addresses to 64 bytes total  */


.section .text
.arm										/* use ARM instructions (not THUMB) here */

.equ	PSR_MODE_USR,	0x10	/* (10000) User Mode (USR) */
.equ	PSR_MODE_FIQ,	0x11	/* (10001) Fast Interrupt Request Mode (FIQ) */
.equ	PSR_MODE_IRQ,	0x12	/* (10010) Interrupt Request Mode (IRQ) */
.equ	PSR_MODE_SVC,	0x13	/* (10011) Supervisor Mode (SVC) */
.equ	PSR_MODE_ABT,	0x17	/* (10111) Abort Mode (ABT) */
.equ	PSR_MODE_UND,	0x1B	/* (11011) Undefined Instruction Mode (UND) */
.equ	PSR_MODE_SYS,	0x1F	/* (11111) System Mode (SYS) */
.equ	PSR_I_BIT,		0x80	/* IRQ disable bit */
.equ	PSR_F_BIT,		0x40	/* FIQ disable bit */


.Lstartup:

				/* switch to SVC mode and setup stack */
				msr		cpsr_c, #(PSR_MODE_SVC|PSR_I_BIT|PSR_F_BIT)
				ldr		r0, =__stack_start__
				mov		sp, r0

				/* Clear .bss section (initialize with zero) */
                ldr		r1, =__bss_start__
                ldr		r2, =__bss_end__
                eor		r0, r0, r0
.Lclearbss:		cmps	r1, r2
                strne	r0, [r1], #4
                bne		.Lclearbss

                /* Initialize .data section (copy from LMA to VMA) */
				ldr		r1, =__data_lma__
                ldr		r2, =__data_start__
                ldr		r3, =__data_end__
.Linitdata:		cmps	r2, r3
				ldrne	r0, [r1], #4
				strne	r0, [r2], #4
				bne		.Linitdata

				/* disable watchdog (that is running by default) */
				/* NOTE: you can only write once */
				ldr		r1, =0xFFFFFD44			/* WDT_MR */
				ldr		r2, [r1]				/* load WDT_MR */
				orr		r2, r2, #0x8000
				str		r2, [r1]

				/* enable IRQ interrupts */
				mrs		r0, cpsr
				bic		r0, r0, #0x80
				msr		cpsr_c, r0

				/* enable FIQ interrupts */
				mrs		r0, cpsr
				bic		r0, r0, #0x40
				msr		cpsr_c, r0

				/* enter main() function */
				mov		fp, sp
				bl		main

				/* prevent us from executing random things */
exit:			b		exit


.Lswi_isr:		stmfd	sp!, {r0-r12, lr}
				ldr		r0, [lr, #-4]
				bic		r0, r0, #0xff000000
				mov		r1, sp
				bl		SWI_Handler
				ldmfd	sp!, {r0-r12, lr}
				mov		pc, lr

